from visual import *
from helix import *

# Gyroscope hanging from a spring
# Bruce Sherwood

top = vector(0,1.,0) # where top of spring is held
ks = 100. # spring stiffness
Lspring = 1. # unstretched length of spring
Rspring = 0.03 # radius of spring
Dspring = 0.03 # thickness of spring wire
Lshaft = 1. # length of gyroscope shaft
Rshaft = 0.03 # radius of gyroscope shaft
M = 1. # mass of gyroscope (massless shaft)
Rrotor = 0.4 # radius of gyroscope rotor
Drotor = 0.1 # thickness of gyroscope rotor
Dsquare = 1.4*Drotor # thickness of square that turns with rotor
I = 0.5*M*Rrotor**2. # moment of inertia of gyroscope
omega = 50 # angular speed of rotor
g = 9.8
Fgrav = vector(0,-M*g,0)

support = box(pos=top+vector(0,0.01,0), size=(0.2,0.02,0.2), color=color.green)
spring = helix(pos=top, axis=(0,-(Lspring+2*M*g/ks),0), coils=10,
                  radius=Rspring, thickness=Dspring, color=(1,1,0))
gyro = frame(pos=top+spring.axis) # gyro.pos at end of spring
shaft = cylinder(frame=gyro, pos=(0,0,0), axis=(Lshaft,0,0),
                 radius=Rshaft, color=(0,1,1))
rotor = cylinder(frame=gyro, pos=0.5*shaft.axis*(Lshaft-Drotor)/Lshaft,
                 axis=norm(shaft.axis)*Drotor,
                 radius=Rrotor, color=(1,0,0))
gyro.axis = norm(shaft.axis)

scene.autoscale = 0
p = vector(0,0,-g*(M*Lshaft/2.)**2/(I*omega)) # make smooth precession
cm = gyro.pos+0.5*Lshaft*gyro.axis # center of mass of shaft
J = I*omega*rotor.axis/Drotor+cross(rotor.pos+0.5*rotor.axis-top,p)
dt = 0.01
t = 0.

while 1:
    # calculate torque of Mg about top of spring
    torque = cross(gyro.pos+0.5*Lshaft*gyro.axis-top,Fgrav)
    # make the approximation of very high spin rate omega
    J = J+torque*dt # total ang. momentum about top of spring
    # spin ang. momentum S is approximately J-(orbital ang. mom.)
    # (neglecting spin ang. mom. components perpendicular to shaft)
    S = J-cross(gyro.pos+0.5*Lshaft*gyro.axis-top,p)
    p = p+(Fgrav-ks*norm(spring.axis)*(mag(spring.axis)-Lspring))*dt
    cm = cm+(p/M)*dt
    gyro.axis = norm(S)
    gyro.pos = cm-0.5*Lshaft*gyro.axis
    gyro.rotate(angle=omega*dt/10.) # easier to see if 0.1 actual omega
    spring.modify(axis=gyro.pos-top)
    t = t+dt
    rate(50)










