from visual import *

# Display extruded text (uppercase only at present)
# By default, display text along x axis, with letters pointing up parallel to y axis
# Bruce Sherwood, Carnegie Mellon University, March 2000

# Example with default values:
#  at = vector(0,0,0) # place to start displaying string
#  at = showstring(at, 'ABC', thickness=0.1,
#    color=currentdisplay.foreground, direction=vector(1.0,0,0), angle=0.0)
# direction vector is direction along which text advances
# angle is counterclockwise rotation angle about x axis
#   before rotating toward direction vector
# Returns pen position for next position to write
# Only uppercase letters at present: others displays as '*'

defaultdir = vector(1.0,0,0)
defaultup = vector(0,1.0,0)

letters = {' ': [(0.39,0,0)],
            'A': [[(0,0,0), (0.13,0,0), (0.37,1.0,0)],
                [(0.88,0,0), (-0.13,0,0),(-0.37,1.0,0)],
                [(0.2,0.3,0), (0.5,0,0), (0,0.1,0)],
                (1.0,0,0)],
            'B': [[(0,0,0), (0.13,0,0), (0,1.0,0)],
                [(0.1,1.0,0), (0.4,0,0), (0,-0.12,0)],
                [(0.1,0.47,0), (0.4,0,0), (0,0.12,0)],
                [(0.1,0,0), (0.4,0,0), (0,0.12,0)],
                [(0.5,1.0,0), (0.26,-0.1,0), (0,-0.12,0)],
                [(0.76,0.9,0), (-0.13,0,0), (0,-0.3,0)],
                [(0.5,0.47,0), (0.26,0.1,0), (0,0.12,0)],
                [(0.5,0.59,0), (0.26,-0.1,0), (0,-0.12,0)],
                [(0.76,0.42,0), (-0.13,0,0), (0,-0.3,0)],
                [(0.5,0,0), (0.26,0.1,0), (0,0.12,0)],
                (0.88,0,0)],
            'C': [[(0,0.2,0), (0.13,0,0), (0,0.6,0)],
                [(0.26,1.0,0), (0.36,0,0), (0,-0.13,0)],
                [(0.26,0,0), (0.36,0,0), (0,0.13,0)],
                [(0.62,1.0,0), (0.26,-0.2,0), (0,-0.13,0)],
                [(0,0.8,0), (0.26,0.2,0), (0,-0.13,0)],
                [(0,0.2,0), (0.26,-0.2,0), (0,0.13,0)],
                [(0.62,0,0), (0.26,0.2,0), (0,0.13,0)],
                (1.05,0,0)],
            'D': [[(0,0,0), (0.13,0,0), (0,1.0,0)],
                [(0.1,1.0,0), (0.4,0,0), (0,-0.12,0)],
                [(0.1,0,0), (0.4,0,0), (0,0.12,0)],
                [(0.5,1.0,0), (0.26,-0.1,0), (0,-0.12,0)],
                [(0.76,0.9,0), (-0.13,0,0), (0,-0.8,0)],
                [(0.5,0,0), (0.26,0.1,0), (0,0.12,0)],
                (1.01,0,0)],
            'E': [[(0,0,0), (0.13,0,0), (0,1.0,0)],
                [(0.1,1.0,0), (0.63,0,0), (0,-0.12,0)],
                [(0.1,0.46,0), (0.58,0,0), (0,0.11,0)],
                [(0.1,0,0), (0.63,0,0), (0,0.12,0)],
                (0.93,0,0)],
            'F': [[(0,0,0), (0.13,0,0), (0,1.0,0)],
                [(0.1,1.0,0), (0.58,0,0), (0,-0.12,0)],
                [(0.1,0.46,0), (0.52,0,0), (0,0.11,0)],
                (0.8,0,0)],
            'G': [[(0,0.2,0), (0.13,0,0), (0,0.6,0)],
                [(0.26,1.0,0), (0.36,0,0), (0,-0.13,0)],
                [(0.26,0,0), (0.36,0,0), (0,0.13,0)],
                [(0.62,1.0,0), (0.26,-0.2,0), (0,-0.13,0)],
                [(0,0.8,0), (0.26,0.2,0), (0,-0.13,0)],
                [(0,0.2,0), (0.26,-0.2,0), (0,0.13,0)],
                [(0.62,0,0), (0.26,0.2,0), (0,0.13,0)],
                [(0.47,0.35,0), (0.44,0,0), (0,0.12,0)],
                [(0.92,0.47,0), (-0.1,0,0), (0,-0.47,0)],
                (1.05,0,0)],           
            'H': [[(0,0,0), (0.13,0,0), (0,1.0,0)],
                [(0.1,0.47,0), (0.6,0,0), (0,0.12,0)],
                [(0.69,0,0), (0.13,0,0), (0,1.0,0)],
                (1.02,0,0)],
            'I': [[(0,0,0), (0.13,0,0), (0,1.0,0)],
                (0.28,0,0)],
            'J': [[(0.43,1.0,0), (0.13,0,0), (0,-0.78,0)],
                [(0,0.35,0), (0,-0.13,0), (0.12,0,0)],
                [(0.2,0,0), (-0.2,0.22,0), (0,0.13,0)],
                [(0.2,0,0), (0.1,0,0), (0,0.13,0)],
                [(0.3,0,0), (0.26,0.22,0), (0,0.13,0)],
                (0.78,0,0)],
            'K': [[(0,0,0), (0.13,0,0), (0,1.0,0)],
                [(0.13,0.36,0), (0.47,0.44,0), (0,0.16,0)],
                [(0.6,0.8,0), (0.2,0.2,0), (-0.18,0,0)],
                [(0.8,0,0), (-0.18,0,0), (-0.4,0.6,0)],
                (0.92,0,0)],
            'L': [[(0,0,0), (0.13,0,0), (0,1.0,0)],
                [(0,0,0), (0.64,0,0), (0,0.13,0)],
                (0.78,0,0)],
            'M': [[(0,0,0), (0.13,0,0), (0,1.0,0)],
                [(0.07,1.0,0), (0.13,0,0), (0.34,-1.0,0)],
                [(0.85,1.0,0), (-0.13,0,0), (-0.31,-1.0,0)],
                [(0.82,0,0), (0.13,0,0), (0,1.0,0)],
                (1.16,0,0)],
            'N': [[(0,0,0), (0.13,0,0), (0,1.0,0)],
                [(0.01,1.0,0), (0.15,0,0), (0.63,-1.0,0)],
                [(0.66,0,0), (0.13,0,0), (0,1.0,0)],
                (1.0,0,0)],
            'O': [[(0,0.2,0), (0.13,0,0), (0,0.6,0)],
                [(0.26,1.0,0), (0.46,0,0), (0,-0.13,0)],
                [(0.26,0,0), (0.46,0,0), (0,0.13,0)],
                [(0.72,1.0,0), (0.26,-0.2,0), (0,-0.13,0)],
                [(0,0.8,0), (0.26,0.2,0), (0,-0.13,0)],
                [(0,0.2,0), (0.26,-0.2,0), (0,0.13,0)],
                [(0.72,0,0), (0.26,0.2,0), (0,0.13,0)],
                [(0.85,0.2,0), (0,0.6,0), (0.13,0,0)],
                (1.13,0,0)],
            'P': [[(0,0,0), (0.13,0,0), (0,1.0,0)],
                [(0.1,1.0,0), (0.4,0,0), (0,-0.12,0)],
                [(0.1,0.47,0), (0.4,0,0), (0,0.12,0)],
                [(0.5,1.0,0), (0.26,-0.1,0), (0,-0.12,0)],
                [(0.76,0.9,0), (-0.13,0,0), (0,-0.3,0)],
                [(0.5,0.47,0), (0.26,0.1,0), (0,0.12,0)],
                (0.85,0,0)],
            'Q': [[(0,0.2,0), (0.13,0,0), (0,0.6,0)],
                [(0.26,1.0,0), (0.46,0,0), (0,-0.13,0)],
                [(0.26,0,0), (0.46,0,0), (0,0.13,0)],
                [(0.72,1.0,0), (0.26,-0.2,0), (0,-0.13,0)],
                [(0,0.8,0), (0.26,0.2,0), (0,-0.13,0)],
                [(0,0.2,0), (0.26,-0.2,0), (0,0.13,0)],
                [(0.72,0,0), (0.26,0.2,0), (0,0.13,0)],
                [(0.85,0.2,0), (0,0.6,0), (0.13,0,0)],
                [(0.53,0.21,0), (0.07,0.08,0), (0.36,-0.28,0)],
                (1.14,0,0)],
            'R': [[(0,0,0), (0.13,0,0), (0,1.0,0)],
                [(0.1,1.0,0), (0.4,0,0), (0,-0.12,0)],
                [(0.1,0.47,0), (0.4,0,0), (0,0.12,0)],
                [(0.5,1.0,0), (0.26,-0.1,0), (0,-0.12,0)],
                [(0.76,0.9,0), (-0.13,0,0), (0,-0.3,0)],
                [(0.5,0.47,0), (0.26,0.1,0), (0,0.12,0)],
                [(0.5,0.59,0), (0.26,-0.1,0), (0,-0.12,0)],
                [(0.76,0.42,0), (-0.13,0,0), (0,-0.3,0)],
                [(0.76,0.12,0), (-0.13,0,0), (0.05,-.12,0)],
                (0.95,0,0)],
            'S': [[(0,0.33,0), (0.12,0,0), (0,-0.13,0)],
                [(0,0.2,0), (0.12,0,0), (0.2,-0.2,0)],
                [(0.2,0,0), (0,0.12,0), (0.38,0,0)],
                [(0.58,0,0), (-0.12,0,0), (0.2,0.2,0)],
                [(0.78,0.2,0), (-0.12,0,0), (0,0.3,0)],
                [(0.78,0.4,0), (0,0.13,0), (-0.73,0.24,0)],
                [(0.05,0.64,0), (0,0.16,0), (0.13,0,0)],
                [(0.05,0.80,0), (0.13,0,0), (0.2,0.2,0)],
                [(0.25,1.0,0), (0.3,0,0), (0,-0.13,0)],
                [(0.55,1.0,0), (-0.13,0,0), (0.2,-0.2,0)],
                (0.93,0,0)],
            'T': [[(0.34,0,0), (0.13,0,0), (0,1.0,0)],
                [(0,1.0,0), (0.8,0,0), (0,-0.12,0)],
                (0.93,0,0)],
            'U': [[(0,1.0,0), (0.13,0,0), (0,-0.78,0)],
                [(0.65,1.0,0), (0.13,0,0), (0,-0.78,0)],
                [(0,0.22,0), (0.26,-0.22,0), (0,0.13,0)],
                [(0.26,0,0), (0.26,0,0), (0,0.13,0)],
                [(0.52,0,0), (0.26,0.22,0), (0,0.13,0)],
                (0.92,0,0)],
            'V': [[(0,1.0,0), (0.15,0,0), (0.36,-1.0,0)],
                [(0.86,1.0,0), (-0.15,0,0), (-0.36,-1.0,0)],
                (0.92,0,0)],
            'W': [[(0,1.0,0), (0.15,0,0), (0.25,-1.0,0)],
                [(0.71,1.0,0), (-0.15,0,0), (-0.25,-1.0,0)],
                [(0.6,1.0,0), (0.15,0,0), (0.25,-1.0,0)],
                [(1.26,1.0,0), (-0.15,0,0), (-0.25,-1.0,0)],
                (1.3,0,0)],
            'X': [[(0.02,1.0,0), (0.17,0,0), (0.68,-1.0,0)],
                [(0,0,0), (0.17,0,0), (0.69,1.0,0)],
                (0.92,0,0)],
            'Y': [[(0.38,0,0), (0.13,0,0), (0,0.4,0)],
                [(0.15,1.0,0), (0,-0.24,0), (0.30,-0.48,0)],
                [(0,1.0,0), (0.15,0,0), (0.15,-0.24,0)],
                [(0.73,1.0,0), (0,-0.24,0), (-0.28,-0.48,0)],
                [(0.88,1.0,0), (-0.15,0,0), (-0.15,-0.24,0)],
                (0.93,0,0)],
            'Z': [[(0,0,0), (0.78,0,0), (0,0.12,0)],
                [(0.05,1.0,0), (0.73,0,0), (0,-0.12,0)],
                [(0,0.12,0), (0.17,0,0), (0.61,0.76,0)],
                (0.83,0,0)],
            '*': [[(0,0.82,0), (0.025,0.07,0), (0.21,-0.06,0)],
                [(0.43,0.82,0), (-0.025,0.07,0), (-0.22,-0.06,0)],
                [(0.17,0.98,0), (0.08,0,0), (0,-0.20,0)],
                [(0.06,0.64,0), (0.06,-0.05,0), (0.12,0.16,0)],
                [(0.36,0.64,0), (-0.06,-0.05,0), (-0.12,0.16,0)],
                (0.53,0,0)]
                }

def makeletterbox(origin, b, xsize, ysize,
                  thickness, color, angle,
                  container):
    barray = array(b)
    ab = barray*array((xsize,ysize,0))
    org = origin+ab[0]
    b = convex(color=color, frame=container)
    if thickness <> 0:
        for i in range(2):
            for j in range(2):
                for k in range(2):
                     b.append(pos=org + i*ab[1] + j*ab[2] + k*vector(0,0,-xsize*thickness))
    else:
        for i in range(2):
            for j in range(2):
                     b.append(pos=org + i*ab[1] + j*ab[2])

def showletter(origin, char, xsize, ysize,
               thickness, color, angle,
               container):
    dx = 1.0
    if letters.has_key(char):
        data = letters[char]
    else:
        data = letters['*']
    for n in range(len(data)-1):
        makeletterbox(origin, data[n], xsize, ysize,
                      thickness, color, angle,
                      container)
    # assume that increment in char description is always of form (dx,0,0)
    dx = data[-1][0]
    return origin+xsize*dx*defaultdir

def showstring(origin, str, size=None, xsize=None, ysize=None,
               thickness=0.1, color=None, angle=0.0, direction=defaultdir):
    direction = norm(vector(direction))
    container = frame(pos=vector(origin), axis=direction)
    if angle <> 0:
        container.rotate(axis=direction, angle=angle)
    if size <> None:
        if xsize == None:
            xsize = size
        if ysize == None:
            ysize = size
    else:
        if xsize == None:
            xsize = 1.0
        if ysize == None:
            ysize = 1.0
    origin = vector(0,0,0)
    for char in str:
        origin = showletter(origin, char, xsize, ysize,
                            thickness, color, angle,
                            container)
    return container.pos+dot(origin,defaultdir)*direction
    
if __name__ == '__main__':
    scene.title = "3D Text"
##    curve(pos=[(0,0,0), (2,0,0)], color=(0.5,0,0))
##    curve(pos=[(0,0,0), (0,2,0)], color=(0,0.5,0))
##    curve(pos=[(0,0,0), (0,0,2)], color=(0,0,0.5))

##    at = (-1,0,-1)
##    str = 'DOG'

    at = (-18,0,0)
    str = 'THE QUICK BROWN FOX JUMPS OVER THE LAZY DOG'
   
    at = showstring(at, str, color=(1,0,0), direction=(1,0,1),
                    thickness=0.3, angle=-pi/12)
    at = showstring(at, 'S', color=color.blue, direction=(1,0,1),
                    thickness=0.3, angle=-pi/12)
